/*
 *  Copyright (C) 2007 Free Software Foundation, Inc.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

	.file "romboot.S"

#define DATA_ADDR	0x8200

	.code16

	.globl  start, _start

start:
_start:

	.word	code_end - start	// It's changed to 0xAA55 in grub-mkrom

image_size:
	.byte	0

	jmp	code_start

checksum:
	.byte	0

hotkey:
	.word	0x3920			// SPACE Key

timeout:
	.word	18 * 2			// in ticks (1/18.2 second)

desc_ofs:
	.byte	desc_start - start

desc_len:
	.byte	desc_end - desc_start

reg_edx:
	.long	0


#ifdef PCI

	. = start + 0x18

	.word	PCIHDR - start		// PCI Header
	.word	PNPHDR - start		// PnP Expansion Header

PCIHDR:
	.ascii	"PCIR"			// Signature
	.word	0			// vendor ID
	.word	0			// device ID
	.word	0			// Pointer to vital product data
	.word	24			// PCI data structure length in bytes
	.byte	0			// PCI data structure revision (0=PCI 2.1)
	.byte	2,0,0			// PCI device class code
	.word	0			// ROM size in 512 byte blocks
	.word	0			// Revision level of code
	.byte	0			// Code type (0=x86 PC-AT)
	.byte	0x80			// Last image indicator
	.word	0			// Reserved

PNPHDR:
	.ascii	"$PnP"			// Signature
	.byte	1			// PnP structure revision
	.byte	2			// PnP structure length in 16 byte blocks
	.word	0			// Offset to next header
	.byte	0			// Reserved
	.byte	0x33			// PnP structure checksum
	.long	0			// Device identifier
	.word	0			// Pointer to manufacturer string
	.word	0			// Pointer to productname string
	.byte	2,0,0			// Device class code
	.byte	0x64			// Device indicators
	.word	0			// Boot connection vector
	.word	0			// Disconnect vector
	.word	0			// Bootstrap entry vector
	.word	0			// Reserved
	.word	0			// Static resource info vector

#endif

desc_start:
	.ascii	"Press SPACE to start GRUB2 ..\0"

	. = desc_start + 32

desc_end:

code_start:
	pushfw
	pushw	%ds
	pushaw

	cld

	pushw	$0
	popw	%ds

	movb	$0x0e, %ah
	xorw	%bx, %bx
	movw	$(desc_start - start), %si

1:
	lodsb	%cs:(%si), %al
	int	$0x10
	cmpb	$0, %al
	jne	1b

	movw	%cs:(timeout - start), %si
	addw	(0x46c), %si

1:

	movb	$0x11, %ah
	int	$0x16
	jnz	1f

	cmpw	(0x46c), %si
	jae	1b

	xorw	%ax, %ax

1:

	cmpw	%cs:(hotkey - start), %ax
	jnz	1f

	movw	$(new_int19 - start),(0x19 * 4)
	movw	%cs, (0x19 * 4 +2)
	jmp	2f

1:
	movw	%ds, %cs:(0)		// ds = 0
2:

1:					// Clear keys
	movb	$1, %ah
	int	$0x16
	jz	1f
	movb	$0, %ah
	int	$0x16
	jmp	1b
1:

	popaw
	popw	%ds
	popfw
	lret

new_int19:
	pushw	%cs
	popw	%ds
	pushw	$(DATA_ADDR >> 4)
	popw	%es

	movzbw	(image_size - start), %cx
	shlw	$9, %cx
	movw	$(code_end - start), %si
	subw	%si, %cx
	xorw	%di, %di

	cld
	rep	movsb

	movl	(reg_edx - start), %edx

	ljmp	$0, $(DATA_ADDR)


code_end:
